<?php
/*
Plugin Name: Redirect URL to Post
Plugin URI: https://chattymango.com/redirect-url-to-post/
Description: Redirects to a post based on parameters in the URL
Author: Chatty Mango
Author URI: https://chattymango.com/
Version: 0.14.1
License: GNU GENERAL PUBLIC LICENSE, Version 3
Text Domain: redirect-url-to-post
*/

// Don't call this file directly
if ( ! defined( 'ABSPATH') ) {
  die;
}

class RedirectUrlToPost
{

  private $redirect_to;

  private $args_filter_or_custom;

  private $args_default_values;

  private $query_args_pass_through_defaults;

  private $query_args_pass_through;

  private $orderby_whitelist;

  private $order_whitelist;

  private $count;

  private $random_bias;

  private $caching_seconds;

  private $verbose_debug;

  private $args_sorting;

  private $php_rand;

  private $headers_sent = false;

  private $debug_mode = 0;

  private $debug_messages;

  /**
  * 	Initial setup: Register the filter and action
  *
  */
  public function __construct() {

    add_action( 'admin_notices', array( $this, 'admin_notice' ) );

    add_action( 'send_headers', array( $this, 'redirect_post_send_headers' ) );

    add_action( 'wp', array( $this, 'redirect_post_id' ) );

    // redirect_to_random_button deprecated since versions after 0.5.2, only kept for backward compatibility
    add_shortcode( 'redirect_to_random_button', array( $this, 'redirect_to_post_button' ) );

    add_shortcode( 'redirect_to_post_button', array( $this, 'redirect_to_post_button' ) );

    add_filter( 'widget_text', 'do_shortcode' );

    if ( is_admin() ) {

      add_filter( 'plugin_action_links_' . plugin_basename( __FILE__ ), array( $this, 'add_help_link' ), 10 );

    }

    if ( isset( $_GET['rutpdebug'] ) ) {

      $this->debug_messages = array();

      $this->debug_mode = (int) $_GET['rutpdebug'];

      if ( 0 == $this->debug_mode ) {

        $this->debug_mode = 1;

      }

      $this->debug_messages[] = "<b>" .

      __( 'Debug mode is on. We will only display the resulting URL, without redirecting.', 'redirect-url-to-post' ) .

      "</b><br>";

    }

  }


  /**
  * Does the actual work - all parts that don't need a post ID
  *
  * @param void
  * @return void
  *
  */
  public function redirect_post_send_headers() {

    global $redirect_post_query_run;

    // Retrieve search criteria from GET query
    // Use sanitized $_GET to be independent from current state of WP Query and possible unavailability of GET parameters.
    if ( ! empty( $_GET['redirect_to'] ) ) {

      // Can use sanitize_key because only small letters and underscores needed
      $this->redirect_to = sanitize_key( $_GET['redirect_to'] );

      // For some parameters we will need to be inside the loop
      if ( 'prev' == $this->redirect_to || 'previous' == $this->redirect_to || 'next' == $this->redirect_to ) {

        return;

      }

      // Prevent being run again when executing the query
      if ( $redirect_post_query_run > 0 ) {

        return;

      }

      $redirect_post_query_run++;

      $this->load_defaults();

      $this->retrieve_get_parameters();

      $this->php_rand = false;

      // Set up the search query depending on the criteria
      switch ( $this->redirect_to ) {

        // Show the latest post
        case 'latest':
        case 'last':

        $this->args_sorting = array(
          'orderby' => 'date',
          'order' => 'DESC',
        );

        break;

        // Show the oldest post
        case 'oldest':
        case 'first':

        $this->args_sorting = array(
          'orderby' => 'date',
          'order' => 'ASC',
        );

        break;

        // Show a random post
        case 'random':

        if ( $this->count > 0 && empty( $this->random_bias ) ) {

          $this->args_sorting = array(
            'orderby'         => 'date',
            'order'           => 'DESC',
            'posts_per_page'  => $this->count
          );

          $this->php_rand = true;

        } elseif ( $this->count < 0 && empty( $this->random_bias ) ) {

          $this->args_sorting = array(
            'orderby'         => 'date',
            'order'           => 'ASC',
            'posts_per_page'  => $this->count
          );

          $this->php_rand = true;

        } else {

          $this->args_sorting = array(
            'posts_per_page'  => -1
          );

          $this->php_rand = true;

        }

        break;

        // find a post based on orderby (and order)
        case 'custom':

        if ( empty( $this->args_filter_or_custom['orderby'] ) ) {

          _e( "Error: The parameter 'custom' requires also 'orderby'", "redirect-url-to-post" );

          exit;

        }


        if ( ! in_array( strtoupper( $this->args_filter_or_custom['order'] ), $this->order_whitelist ) ) {

          _e( "Error: Unrecognized value of parameter 'order'", "redirect-url-to-post" );

          exit;

        }


        if ( ! in_array( $this->args_filter_or_custom['orderby'], $this->orderby_whitelist ) ) {

          _e( "Error: Unrecognized value of parameter 'orderby'", "redirect-url-to-post" );

          exit;

        }


        $this->args_sorting = array();

        break;

        default:

        /**
        * Unrecognized value of 'redirect_to' => finish processing and let other plugins do their work
        */
        return;

        break;
      }

      $this->find_post();

    }

  }



  /**
  * Does the actual work - all parts that need a post ID
  *
  * @param void
  * @return void
  */
  public function redirect_post_id() {

    global $redirect_post_query_run, $wp_query;

    // Retrieve search criteria from GET query
    // Use sanitized $_GET to be independent from current state of WP Query and possible unavailability of GET parameters.
    if ( ! empty( $_GET['redirect_to'] ) ) {

      // Can use sanitize_key because only small letters and underscores needed
      $this->redirect_to = sanitize_key( $_GET['redirect_to'] );

      // Return if we are dealing with parameters that should have been processed earlier
      if ( 'prev' != $this->redirect_to && 'previous' != $this->redirect_to && 'next' != $this->redirect_to ) {

        return;

      }

      // Prevent being run again when executing the query
      if ( $redirect_post_query_run > 0 ) {

        return;

      }

      $redirect_post_query_run++;

      $this->headers_sent = true; // It is possible that at this stage some plugin has already sent a header

      $this->load_defaults();

      $this->retrieve_get_parameters();

      if ( is_home() ) {

        $page_id = get_option('page_for_posts');

        $post_date = get_the_date( '', $page_id ); // virtual front page for blog posts has date "0"

        $post_type = 'page';

      } else {

        if ( empty( $wp_query ) || empty( $wp_query->post ) ) { // something terrible has happened

          if ( defined( 'WP_DEBUG' ) && WP_DEBUG ) {

            error_log( '[Redirect URL to Post] Cannot identify the post.' );

          }

          if (  ! empty( $post_ids ) && $this->debug_mode ) {

            $this->debug_messages[] = __( 'Cannot identify the post.', 'redirect-url-to-post' );

          }

          $this->output_debug_messages();

          return; // steal away and pretend to be invisible

        }

        $post_date = $wp_query->post->post_date;

        $post_type = $wp_query->post->post_type;

      }

      $date_query_items = array();

      // Set up the search query depending on the criteria
      switch ( $this->redirect_to ) {

        // Go to the previous post
        case 'previous':
        case 'prev':

        $this->args_sorting = array(
          'orderby' => 'date',
          'order' => 'DESC',
        );

        $date_query_items['before'] = $post_date;

        $this->args_filter_or_custom['post_type'] = $post_type;

        break;

        // Go to the next post
        case 'next':

        $this->args_sorting = array(
          'orderby' => 'date',
          'order' => 'ASC',
        );

        $date_query_items['after'] = $post_date;

        $this->args_filter_or_custom['post_type'] = $post_type;

        break;

        default:

        /**
        * Unrecognized value of 'redirect_to' => finish processing and let other plugins do their work
        */
        return;

        break;
      }

      /**
      * Construct the 'date_query' array
      */
      if ( ! empty( $date_query_items ) ) {

        $this->args_filter_or_custom['date_query'] = array(
          $date_query_items
        );

      }

      $this->find_post();

    }

  }


  /**
  * Loads all default values for the query
  *
  * @param void
  * @return void
  */
  public function load_defaults()
  {

    $this->args_filter_or_custom = array(
      'fields'              => 'ids',
      'order'               => 'DESC',
      'post_type'           => 'post',
      'suppress_filters'    => true,
      'ignore_sticky_posts' => true,
      'posts_per_page'      => 1,
      'post_status'         => 'publish'
    );


    /**
    *  Post filters
    *
    *  Get standard parameters that make sense to use
    */
    $this->args_default_values = array(
      'cat', // strangely 'category' not working
      'category_name',
      'date_query_after', // alias for after
      'after', // translates to date_query construct
      'date_query_before', // alias for before
      'before', // translates to date_query construct
      'day',
      'hour',
      'ignore_sticky_posts',
      'minute',
      'monthnum',
      'offset',
      'order',
      'orderby',
      'post_type', // requires has_archive for that post_type
      's',
      'second',
      'suppress_filters',
      'tag',
      'w',
      'author',
      'author_name',
      'has_password',
      'tag_id',
      'exclude'
    );


    /**
    * Get parameters that don't affect the retrieval of posts and that will be passed through to the final query
    */
    $this->query_args_pass_through_defaults = array(
      'utm_source', // Google Analytics
      'utm_campaign', // Google Analytics
      'utm_medium', // Google Analytics
      'utm_term', // Google Analytics
      'utm_content', // Google Analytics
      'pk_campaign', // Matomo Analytics
      'pk_kwd', // Matomo Analytics
      'pk_source', // Matomo Analytics
      'pk_medium', // Matomo Analytics
      'pk_content', // Matomo Analytics
    );

    // Whitelisted orderby parameters
    $this->orderby_whitelist = array(
      'author',
      'comment_count',
      'date',
      'ID',
      'menu_order',
      'modified',
      'name',
      'none',
      'parent',
      'rand',
      'title',
      'type',
    );

    // Whitelisted order parameter
    $this->order_whitelist = array(
      'ASC',
      'DESC'
    );

  }


  /**
  * Read the GET parameters
  *
  *
  * @param void
  * @return void
  */
  public function retrieve_get_parameters()
  {

    foreach ( $this->args_default_values as $value ) {

      if ( isset( $_GET[ $value ] ) &&  '' !== $_GET[ $value ] ) {

        if ( 'exclude' == $value ) {

          $this->args_filter_or_custom['post__not_in'] = array_map( 'intval', array_map( 'trim', explode( ',', $_GET['exclude'] ) ) );

        } elseif ( 'offset' == $value ) {

          $this->args_filter_or_custom['offset'] = abs( intval( $_GET['offset'] ) );

        } elseif ( 'suppress_filters' == $value ) {

          $this->args_filter_or_custom['suppress_filters'] = $_GET['suppress_filters'] ? true : false;

        } else {

          // Sanitized with sanitize_text_field because some values may be uppercase or spaces
          $this->args_filter_or_custom[ $value ] = sanitize_text_field( $_GET[ $value ] );

        }

      }

    }

    /**
    * Special processing for 'date_query_before' and 'date_query_after'
    */
    $date_query_items = array();

    if ( ! empty( $this->args_filter_or_custom['date_query_before'] ) ) {

      $date_query_items['before'] = $this->args_filter_or_custom['date_query_before'];

      unset( $this->args_filter_or_custom['date_query_before'] );

    }

    if ( ! empty( $this->args_filter_or_custom['before'] ) ) {

      $date_query_items['before'] = $this->args_filter_or_custom['before'];

      unset( $this->args_filter_or_custom['before'] );

    }

    if ( ! empty( $this->args_filter_or_custom['date_query_after'] ) ) {

      $date_query_items['after'] = $this->args_filter_or_custom['date_query_after'];

      unset( $this->args_filter_or_custom['date_query_after'] );

    }

    if ( ! empty( $this->args_filter_or_custom['after'] ) ) {

      $date_query_items['after'] = $this->args_filter_or_custom['after'];

      unset( $this->args_filter_or_custom['after'] );

    }

    /**
    * Construct the 'date_query' array
    */
    if ( ! empty( $date_query_items ) ) {

      $this->args_filter_or_custom['date_query'] = array(
        $date_query_items
      );

    }


    if ( ! empty( $_GET['count'] ) ) {

      $this->count = intval( $_GET['count'] );

    } else {

      $this->count = 0;

    }

    if ( ! empty( $_GET['bias'] ) ) {

      $this->random_bias = intval( $_GET['bias'] );

    } else {

      $this->random_bias = false;

    }


    // caching
    // We can override the settings with a constant so that visitors cannot maliciously overuse the database.
    if ( defined( 'CHATTY_MANGO_RUTP_CACHE' ) ) {

      $this->caching_seconds = intval( CHATTY_MANGO_RUTP_CACHE );

    } else {

      if ( isset( $_GET['cache'] ) ) {

        $this->caching_seconds = intval( $_GET['cache'] );

      } else {

        $this->caching_seconds = 60;

      }

    }

    if ( isset( $_GET['comment_count'] ) ) {

      $compare = ( substr( $_GET['comment_count'], 0, 1 ) == '-' ) ? '!=' : '=';

      $this->args_filter_or_custom['comment_count'] = array(
        'value' => abs( intval( $_GET['comment_count'] ) ),
        'compare' => $compare,
      );

    } elseif ( isset( $_GET['comment_count_min'] ) ) {

      $this->args_filter_or_custom['comment_count'] = array(
        'value' => abs( intval( $_GET['comment_count_min'] ) ),
        'compare' => '>=',
      );

    } elseif ( isset( $_GET['comment_count_max'] ) ) {

      $this->args_filter_or_custom['comment_count'] = array(
        'value' => abs( intval( $_GET['comment_count_max'] ) ),
        'compare' => '<=',
      );

    }


    if ( isset( $_GET['verbose_debug'] ) ) {

      $this->verbose_debug = true;

    } else {

      $this->verbose_debug = false;

    }



    foreach ( $this->query_args_pass_through_defaults as $value ) {

      if ( isset( $_GET[ $value ] ) ) {

        // Sanitized with sanitize_text_field because some values may be uppercase or spaces
        $this->query_args_pass_through[ $value ] = sanitize_text_field( $_GET[ $value ] );

      }

    }

  }


  /**
  * Finds the post that matches the parameters
  *
  *
  * @param void
  * @return void
  */
  public function find_post()
  {

    if ( isset( $this->args_sorting ) ) {

      $args = array_merge( $this->args_filter_or_custom, $this->args_sorting );

    } else {

      $args = $this->args_filter_or_custom;

    }


    if ( ! empty( $this->random_bias ) && ! empty( $args['offset'] ) ) {

      /**
      * We'll have to apply the offset manually
      */
      $offset = $args['offset'];

      unset( $args['offset'] );

    }

    /**
    * Sort by key to avoid duplicate cache for identical queries
    */
    ksort( $args );


    if ( $this->debug_mode > 1 ) {

      $display_args = $args;

      // remove what doesn't depend on URL parameters
      unset( $display_args['fields'] );

      unset( $display_args['posts_per_page'] );

      unset( $display_args['post_status'] );

      $this->debug_messages[] = __( 'Query parameters:', 'redirect-url-to-post' );

      $this->debug_messages[] = "<pre>" . print_r( $display_args, true ) . "</pre>\n";

    }

    /**
    * Retrieve the post and redirect to its permalink
    */
    if ( ! empty( $this->caching_seconds ) ) {

      $key_cache = md5( serialize( $args ) . serialize( $this->php_rand ) );

      /**
      * We save every $key_cache as own transient so that they can have different lifetimes.
      */
      $post_ids = get_transient( 'chatty_mango_rutp_post_ids-' . $key_cache );

      if ( ! empty( $post_ids ) && defined( 'WP_DEBUG' ) && WP_DEBUG && $this->verbose_debug ) {

        error_log( sprintf( '[Redirect URL to Post] We found something useful in the cache for redirect_to=%s.', $this->redirect_to ) );

      }

      if (  ! empty( $post_ids ) && $this->debug_mode ) {

        $this->debug_messages[] = sprintf( __( 'We found %d matching post(s) in the cache.', 'redirect-url-to-post' ), count( $post_ids ) ) ;

      }

    }

    if ( empty( $post_ids ) ) {

      /**
      * We have to query the database
      */

      wp_reset_postdata();

      /**
      * Remove all hooks that might cancel out our 'orderby'
      */
      remove_all_actions( 'pre_get_posts' );

      /**
      * Reduce the risk of interference from other plugins
      * (It can be turned off, for example on multilingual websites)
      */
      if ( $this->args_filter_or_custom['suppress_filters'] ) {

        remove_all_filters( 'posts_clauses' );
        remove_all_filters( 'posts_orderby' );
        remove_all_filters( 'posts_where' );
        remove_all_filters( 'posts_join' );
        remove_all_filters( 'posts_groupby' );

      }

      /**
      * WP_Query is supposed to sanitize $args
      */
      $the_query = new WP_Query( $args );

      $post_ids = $the_query->posts;

      if (  ! empty( $post_ids ) && $this->debug_mode ) {

        $this->debug_messages[] = sprintf( __( 'We found %d matching post(s) in the database.', 'redirect-url-to-post' ), count( $post_ids ) ) ;

      }

      if ( ! empty( $post_ids ) && ! empty( $key_cache ) ) {

        /**
        * Save the result to the transient cache
        */

        if ( defined( 'WP_DEBUG' ) && WP_DEBUG && $this->verbose_debug ) {

          error_log( sprintf( '[Redirect URL to Post] We filled the cache for redirect_to=%s with a lifetime of %d seconds.', $this->redirect_to, $this->caching_seconds ) );

        }

        if ( $this->debug_mode ) {

          $this->debug_messages[] = sprintf( __( 'We filled the cache with a lifetime of %d seconds.', 'redirect-url-to-post' ), $this->caching_seconds ) ;

        }

        set_transient( 'chatty_mango_rutp_post_ids-' . $key_cache, $post_ids, $this->caching_seconds );

      }

    }

    if ( ! empty( $post_ids ) ) {

      if ( $this->php_rand ) {

        if ( count( $post_ids ) ) {

          if ( empty( $this->random_bias ) ) {

            /**
            * Pick a random post from the array
            */
            $post_id = $post_ids[ array_rand( $post_ids ) ];

          } else {

            /**
            * Get the subsection of posts that will receive the bias
            */
            if ( $this->count > 0 ) {

              if ( ! empty( $offset ) ) {

                $start = $offset;

              } else {

                $start = 0;

              }

            } else {

              if ( ! empty( $offset ) ) {

                $start = $this->count - $offset; // $this->count is negative

              } else {

                $start = $this->count;

              }

            }

            $bias_post_ids = array_slice( $post_ids, $start, abs( $this->count ) );

            if ( rand( 0, 99 ) < $this->random_bias ) {
              /**
              * Pick a post from $bias_post_ids
              */
              $post_id = $bias_post_ids[ array_rand( $bias_post_ids ) ];

            } else {
              /**
              * Pick a post from the rest
              */
              $post_ids_diff = array_diff( $post_ids, $bias_post_ids );

              $post_id = $post_ids_diff[ array_rand( $post_ids_diff ) ];

            }

          }

        }

        if (  $this->debug_mode ) {

          $this->debug_messages[] = __( 'We picked a random post.', 'redirect-url-to-post' );

        }

      } else {

        /**
        * Use the first number of the array
        */
        $post_id = reset( $post_ids );

      }

    }

    if ( isset( $post_id ) ) {

      $permalink = get_permalink( $post_id );

      /**
      * Add query parameters that appear in the final URL
      */

      if ( ! empty( $this->query_args_pass_through ) ) {

        $permalink = esc_url_raw( add_query_arg( $this->query_args_pass_through, $permalink ) );

      }

      $this->redirect( $permalink );

    } else {

      // Nothing found, go to post with id as specified by redirect_to_default, or home

      if ( isset( $_GET['default_redirect_to'] ) ) {

        $default_redirect_to = sanitize_key( $_GET['default_redirect_to'] );

      }

      if ( empty( $default_redirect_to ) ) {

        if ( defined( 'WP_DEBUG' ) && WP_DEBUG && $this->verbose_debug ) {

          error_log( '[Redirect URL to Post] Nothing found, going to site URL' );

        }

        if ( $this->debug_mode ) {

          $this->debug_messages[] = __( 'Nothing found, going to site URL.', 'redirect-url-to-post' );

        }

        // no default given => go to home page
        $this->redirect( site_url() );

      } else {

        $permalink = get_permalink( $default_redirect_to );

        if ( $permalink === FALSE ) {

          if ( defined( 'WP_DEBUG' ) && WP_DEBUG && $this->verbose_debug ) {

            error_log( '[Redirect URL to Post] Nothing found and default_redirect_to not found, going to site URL.' );

          }

          if ( $this->debug_mode ) {

            $this->debug_messages[] = __( 'Nothing found and default_redirect_to not found, going to site URL.', 'redirect-url-to-post' );

          }

          // default post or page does not exist => go to home page
          $this->redirect( site_url() );

        } else {

          if ( defined( 'WP_DEBUG' ) && WP_DEBUG && $this->verbose_debug ) {

            error_log( '[Redirect URL to Post] Nothing found, going to default_redirect_to.' );

          }

          if ( $this->debug_mode ) {

            $this->debug_messages[] = __( 'Nothing found, going to default_redirect_to.', 'redirect-url-to-post' );

          }

          $this->redirect( $permalink );

        }

      }

    }

  }


  /**
  * Displays an admin notice
  *
  *
  * @param void
  * @return void
  */
  public function admin_notice()
  {
    /**
    * Display only for first-time activation, not after updates
    * Don't use on_activation, because then you cannot use localization
    */
    if ( get_option( 'redirect-url-to-post-onboarding', false ) === false ) {

      $plugin_link = 'https://chattymango.com/redirect-url-to-post/';

      $documentation_link = 'https://documentation.chattymango.com/documentation/redirect-url-to-post/';

      $random_post_link = get_site_url( null, '/?redirect_to=random' );
      $latest_post_link = get_site_url( null, '/?redirect_to=latest' );
      $date_query_before_post_link = get_site_url( null, '/?redirect_to=latest&before=10%20minute%20ago' );
      $date_query_after_post_link = get_site_url( null, '/?redirect_to=random&after=1%20month%20ago&cache=20' );

      $html = '<div class="notice notice-info is-dismissible"><p>' .
      '<h3>' . sprintf( __( 'Thank you for installing Redirect URL to Post!', 'redirect-url-to-post' ), 'href="' . $plugin_link . '?pk_campaign=rutp&pk_kwd=onboarding" target="_blank"' ) . '</h3>' .
      '<p>' . __( "This plugin doesn't have any settings. You configure it entirely through the URL query parameters.", 'redirect-url-to-post' ) . '</p>' .
      '<h4>' . __( 'Here are some examples:', 'redirect-url-to-post' ) . '</h4>' .
      '<ul style="list-style-type:disc; padding-left: 20px;">
      <li>' . sprintf( __( 'Go to a random post: <a %1$s>%2$s</a>', 'redirect-url-to-post' ), 'href="' . $random_post_link . '" target="_blank"', $random_post_link ) . '</li>
      <li>' . sprintf( __( 'Go to your latest post: <a %1$s>%2$s</a>', 'redirect-url-to-post' ), 'href="' . $latest_post_link . '" target="_blank"', $latest_post_link ) . '</li>
      <li>' . sprintf( __( 'Go to your latest post that was created at least 10 minutes ago: <a %1$s>%2$s</a> (We replaced all spaces by "%%20".)', 'redirect-url-to-post' ), 'href="' . $date_query_before_post_link . '" target="_blank"', $date_query_before_post_link ) . '</li>
      <li>' . sprintf( __( 'Go to a random post from the past month and use the cache: <a %1$s>%2$s</a> (or to the front page, if nothing was found)', 'redirect-url-to-post' ), 'href="' . $date_query_after_post_link . '" target="_blank"', $date_query_after_post_link ) . '</li>
      </ul>' .
      '<p>' . __( 'You can use these links anywhere in your posts and pages, in menus or in buttons. You can even send them in a newsletter. Or you could try the shortcode [redirect_to_post_button] in a page or post.', 'redirect-url-to-post' ) . '</p>' .
      '<p>' . sprintf( __( 'Please find a description of all parameters in the <a %s>plugin documentation</a>.', 'redirect-url-to-post' ), 'href="' . $documentation_link . '?pk_campaign=rutp&pk_kwd=onboarding" target="_blank"' ) . '</p>' .
      '<p>' . __( 'Have fun!', 'redirect-url-to-post' ) . '</p>' .
      '</p></div><div clear="all" /></div>';

      echo $html;

      update_option( 'redirect-url-to-post-onboarding', 1 );

    }

  }


  /**
  * Redirect to a URL
  *
  * Using own redirection so that we can add a header.
  * Firefox 57 needs to be told not to cache.
  *
  * @param string $link Permalink where to redirect to
  * @return void
  */
  private function redirect( $link ) {

    if ( $this->debug_mode ) {

      $this->debug_messages[] = sprintf( '<br><b>=> ' . __( 'The resulting URL is: <a href="%s" target="_blank">%s</a>', 'redirect-url-to-post' ) . "</b><br>", $link, $link );

      $this->debug_messages[] = sprintf( __( 'For more help visit the <a href="%s" target="_blank">documentation</a>.', 'redirect-url-to-post' ), 'https://documentation.chattymango.com/documentation/redirect-url-to-post/?pk_campaign=rutp&pk_kwd=debug' );

      $this->output_debug_messages();

      exit;

    }

    if ( $this->headers_sent ) {

      wp_redirect( $link, 307 );

    } else {

      /**
      * best experience with code 307 for preventing caching in browser
      * create own redirect to be able to add own header
      */
      header( 'Cache-Control: no-cache, must-revalidate' );
      header( 'Location: ' . $link, true, 307 );

    }

    exit;

  }


  /**
  * Displays the debug messages on the screen
  *
  * @param void
  * @return void
  */
  public function output_debug_messages()
  {

    echo implode( "<br>\n", $this->debug_messages );

  }


  /**
  * Add Help link to plugins page
  *
  * @param array $links
  * @return array
  */
  public function add_help_link( $links ) {

    $settings_link = '<a href="https://documentation.chattymango.com/documentation/redirect-url-to-post/?pk_campaign=rutp&pk_kwd=settings" target="_blank">' . __( "Help", "redirect-url-to-post" ) . '</a>';

    array_unshift( $links, $settings_link );

    return $links;

  }


  /**
  * Implements a shortcode to add a random button
  * Requires Javascript
  *
  * @param array $atts Shortcode parameters
  * @return string HTML to replace the shortcode
  */
  public function redirect_to_post_button( $atts = array() ) {

    extract( shortcode_atts( array(
      'div_class' => null,
      'button_class' => null,
      'text' => __( 'Random Post', 'redirect-url-to-post' ),
      'redirect_to' => 'random',
      'params' => null
    ), $atts ) );

    $url = get_site_url( null, '/?redirect_to=' . $redirect_to );

    /**
    * Add optional paramters
    */
    if ( ! empty( $params ) ) {

      if ( strpos( $params, '=>' ) !== false ) {

        // old format

        // split the string by commas
        $params_array = explode( ',', $params );

        foreach ( $params_array as $item ) {

          $item_array = explode( '=>', $item );

          if ( count( $item_array) == 2 ) {

            $item_array[0] = sanitize_key( trim( $item_array[0] ) );
            $item_array[1] = sanitize_title( trim( $item_array[1] ) );

            $url .= '&' . $item_array[0] . '=' . $item_array[1];

          }

        }

      } else {

        // split the string by |
        $params_array = explode( '|', $params );

        foreach ( $params_array as $item ) {

          $item_array = explode( '=', $item );

          if ( count( $item_array) == 2 ) {

            $item_array[0] = sanitize_key( trim( $item_array[0] ) );
            $item_array[1] = sanitize_title( trim( $item_array[1] ) );

            $url .= '&' . $item_array[0] . '=' . $item_array[1];

          }

        }

      }

    }

    $html = '<div';

    if ( ! empty( $div_class ) ) {

      $html .= ' class="' . sanitize_html_class( $div_class ) . '"';

    }

    $html .= '>';

    /**
    * Construct the actual button
    */
    $html .= '<button';

    if ( ! empty( $button_class ) ) {

      $html .= ' class="' . sanitize_html_class( $button_class ) . '"';

    }

    $url = str_replace( '"', '\"', $url );

    $url = str_replace( "'", '\"', $url );

    $html .= ' onclick="window.location.href=\'' . $url . '\'">' . sanitize_text_field( $text ) . '</button>';

    $html .= '</div>';

    return $html;

  }


  /**
  * Display some help on plugin activation
  *
  * @param void
  * @return void
  */
  public function on_activation() {}


  }



  /**
  * launch the plugin: add actions and filters
  */
  $redirectUrlToPost = new RedirectUrlToPost();


  /**
  * Internationalization before WP 4.6
  *
  * @param void
  * @return void
  */
  function redirect_url_to_post_load_plugin_textdomain() {

    load_plugin_textdomain( 'redirect-url-to-post', FALSE, dirname( plugin_basename( __FILE__ ) ) . '/languages/' );

  }

  add_action( 'init', 'redirect_url_to_post_load_plugin_textdomain' );
